// Filename: resitele.c
// Name of author: Petr Kovar
// Work begun:    01/12/2006
// Last modified: 17/06/2007
//
// Description:
// file with program for generating tables for homel.vsb.cz/~kov16/ulohy

/*
Copyright (C) 2006 Petr Kov
You can reach me at Petr.Kovar@vsb.cz

-------------------------------------

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

// ------------------------------------------------------------------------------------------------------------------------

// Standardn knihovny
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <stdint.h>
#include <ctype.h>

// Definice typ
#include "resitele_types.h"
#include "commandline.h"
#include "resitele.h"
#include "messages.h"

// Dodlat: 
// - counting sort pro uspodn eench loh (kadho eitele)
// - promazat kd : exit

// Tabulka mesicu
char * MESIC[] = {"pedleden", "leden", "nor", "bezen", "duben", "kvten", "erven", "ervenec", "srpen", "z", "jen", "listopad", "prosinec", "poprosinec"};

// Alokuj jedno nov een njak lohy konkrtnho eitele (seznam pro kadho eitele)
RESENA_ULOHA * new_resena_uloha(RESENI * p_reseni) {
  RESENA_ULOHA * p_resena_uloha;               // nov struktura RESENA_ULOHA jednoho eitele

  // alokuj pam
  p_resena_uloha = (RESENA_ULOHA *) malloc(sizeof(RESENA_ULOHA));
  // inicializan hodnoty
  p_resena_uloha->p_reseni = p_reseni;         // pointer na een tto lohy
  p_resena_uloha->p_next_resena_uloha = NULL;  // pointer na dal eenou lohu tohoto eitele
  // pointer na novou alokovanou strukturu
  return p_resena_uloha;
}

// Vymae seznam a uvoln pam vech een konkrtnho eitele
STATUS free_resena_uloha_list(RESITEL * p_resitel) {
  RESENA_ULOHA * p_resena_uloha;                       // pointer na eenou lohu
  RESENA_ULOHA * p_next_resena_uloha;                  // pointer na dal eenou lohu

  // projdi cel seznam
  p_resena_uloha = p_resitel->p_first_resena_uloha;
  while (p_resena_uloha != NULL) {
    p_next_resena_uloha = p_resena_uloha->p_next_resena_uloha;
    // uvoln strukturu dal lohy
    free(p_resena_uloha);
    // move on
    p_resena_uloha = p_next_resena_uloha;
  }
  return OK;
}

// Alokuj jedno nov konkrtn een tto lohy (seznam pro lohu)
RESENI_ULOHY * new_reseni_ulohy(RESENI * p_reseni) {
  RESENI_ULOHY * p_reseni_ulohy;         // nov struktura RESENI_ULOHY njak lohy

  // alokuj pam
  p_reseni_ulohy = (RESENI_ULOHY *) malloc(sizeof(RESENI_ULOHY));
  // inicializan hodnoty
  p_reseni_ulohy->p_reseni = p_reseni;         // pointer na een tto lohy
  p_reseni_ulohy->p_next_reseni_ulohy = NULL;  // pointer na dal een tto lohy
  // pointer na novou alokovanou strukturu
  return p_reseni_ulohy;
}

// Vymae seznam a uvoln pam vech een konkrtn lohy
STATUS free_reseni_ulohy_list(ULOHA * p_uloha) {
  RESENI_ULOHY * p_reseni_ulohy;         // pointer na een jedn lohy
  RESENI_ULOHY * p_next_reseni_ulohy;    // pointer na dal een tto jedn lohy

  // projdi cel seznam
  p_reseni_ulohy = p_uloha->p_first_reseni_ulohy;
  while (p_reseni_ulohy != NULL) {
    p_next_reseni_ulohy = p_reseni_ulohy->p_next_reseni_ulohy;
    // uvoln strukturu odkazu na dal een lohy
    free(p_reseni_ulohy);
    // move on
    p_reseni_ulohy = p_next_reseni_ulohy;
  }
  return OK;
}

// Alokuj novou lohu (v archivu)
ULOHA * new_uloha(ARCHIV * p_archiv, CISLO index_ulohy, JMENO_ULOHY jmeno_ulohy, BODY body_ulohy) {
  ULOHA * p_uloha;                       // nov struktura ULOHA

  // alokuj pam
  p_uloha = (ULOHA *) malloc(sizeof(ULOHA));
  // inicializan hodnoty
  p_uloha->p_archiv = p_archiv;          // pointer na archiv, ve kterm je loha
  p_uloha->index_ulohy = index_ulohy;    // slo (index) lohy jako slo
  p_uloha->radek_souboru = 0;            // slo dku souboru, zatm nen
  p_uloha->jmeno_ulohy = jmeno_ulohy;    // jmno tto lohy
  p_uloha->body_ulohy = body_ulohy;      // maximln poet bod za lohu
  p_uloha->p_next_uloha = NULL;          // pointer na dal lohu
  p_uloha->p_first_reseni_ulohy = NULL;  // prvn een tto lohy
  p_uloha->n_reseni = 0;                 // poet een tto 'lohy'
  p_uloha->n_uplnych_reseni = 0;         // poet plnch een tto 'lohy'
  // pointer na novou alokovanou strukturu
  return p_uloha;
}

// Uvoln pam jedn lohy
STATUS free_uloha(ULOHA * p_uloha) {
  if (p_uloha != NULL) {
    // nejprve podstruktury
    if (p_uloha->jmeno_ulohy != NULL) {
      free(p_uloha->jmeno_ulohy);
    }
    // seznam vech een tto lohy
    free_reseni_ulohy_list(p_uloha);
    // uvoln strukturu
    free(p_uloha);
  }
  return OK;
}

// Alokuj nov archiv
ARCHIV * new_archiv(TYP typ_jmena_archivu, JMENO jmeno_archivu, CISLO cislo_archivu) {
  ARCHIV * p_archiv;                                // nov struktura ARCHIV
  JMENO jmeno_archivu_kopie = NULL;                 // kopie jmna archivu

  // alokuj pam
  p_archiv = (ARCHIV *) malloc(sizeof(ARCHIV));
  // inicializan hodnoty
  p_archiv->typ_jmena_archivu = typ_jmena_archivu;  // jak je uloeno slo archivu? Zatm nen!
  jmeno_archivu_kopie = (JMENO) malloc(sizeof(char)*(strlen(jmeno_archivu)+1));
  strcpy(jmeno_archivu_kopie, jmeno_archivu);       // lokln kopie
  p_archiv->jmeno_archivu = jmeno_archivu_kopie;    // jmno archivu jako string
  p_archiv->cislo_archivu = cislo_archivu;          // jmno archivu jako slo
  p_archiv->p_first_uloha = NULL;                   // pointer na prvn 'loha'
  p_archiv->n_uloh_v_archivu = 0;                   // zatm adn loha v archivu
  p_archiv->p_next_archiv = NULL;                   // pointer na dal archiv
  // pointer na novou alokovanou strukturu
  return p_archiv;
}

// Uvoln pam jednoho archivu (a loh v nm)
STATUS free_archiv(ARCHIV * p_archiv) {
  ULOHA * p_uloha;                       // pointer na lohu
  ULOHA * p_next_uloha;                  // pointer na dal lohu

  if (p_archiv != NULL) {
    // nejprve podstruktury
    //D    if ((p_archiv->typ_jmena_archivu == TYP_CISLA_STR) && (p_archiv->jmeno_archivu != NULL)){
    if (p_archiv->jmeno_archivu != NULL) {
      free(p_archiv->jmeno_archivu);
    }
    p_uloha = p_archiv->p_first_uloha;
    while (p_uloha != NULL) {
      p_next_uloha = p_uloha->p_next_uloha;
      // uvoln strukturu dal lohy
      free_uloha(p_uloha);
      // move on
      p_uloha = p_next_uloha;
    }
    // uvoln strukturu
    free(p_archiv);
  }
  return OK;
}

// Alokuj nov seznam archiv (a loh v nich)
SEZNAM_ULOH * new_seznam_uloh(void) {
  SEZNAM_ULOH * p_seznam_uloh;          // nov struktura SEZNAM_ULOH

  // alokuj pam
  p_seznam_uloh = (SEZNAM_ULOH *) malloc(sizeof(SEZNAM_ULOH));
  // inicializan hodnoty
  p_seznam_uloh->p_first_archiv = NULL; // jet dn archiv
  p_seznam_uloh->n_archivu = 0;         // dn archivy...
  p_seznam_uloh->n_uloh = 0;            // ... dn lohy
  // pointer na novou alokovanou strukturu
  return p_seznam_uloh;
}

// Uvoln alokovanou pam seznamu loh
STATUS free_seznam_uloh(SEZNAM_ULOH * p_seznam_uloh) {
  ARCHIV * p_archiv;                    // pointery na archiv
  ARCHIV * p_next_archiv;               // pointery na dal archiv

  if (p_seznam_uloh != NULL) {
    // nejprve podstruktury
    p_archiv = p_seznam_uloh->p_first_archiv;
    while (p_archiv != NULL) {
      p_next_archiv = p_archiv->p_next_archiv;
      // uvoln strukturu archivu
      free_archiv(p_archiv);
      // move on
      p_archiv = p_next_archiv;
    }
    // uvoln strukturu
    free(p_seznam_uloh);
  }
  return OK;
}

// Alokuj novho eitele
RESITEL * new_resitel(JMENO jmeno_resitele) {
  RESITEL * p_resitel;                           // nov struktura RESITEL
  JMENO jmeno_resitele_kopie;                    // kopie jmna eitele

  // alokuj pam
  p_resitel = (RESITEL *) malloc(sizeof(RESITEL));
  // inicializan hodnoty
  jmeno_resitele_kopie = (JMENO) malloc(sizeof(char)*(strlen(jmeno_resitele)+1));
  strcpy(jmeno_resitele_kopie, jmeno_resitele);  // lokln kopie
  p_resitel->jmeno_resitele = jmeno_resitele_kopie;  // cel jmno 'eitele' jako string
  p_resitel->suma_bodu = 0;                      // celkov poet bod
  p_resitel->pocet_uloh = 0;                     // celkov poet vyeench loh
  p_resitel->pocet_plnych_uloh = 0;              // poet vyeench loh za pln poet bod
  p_resitel->p_first_resena_uloha = NULL;        // pointer na prvn vyeenou lohu
  p_resitel->p_next_resitel = NULL;              // pointer na dalho eitele
  // pointer na novou alokovanou strukturu
  return p_resitel;
}

// Uvoln pam jednoho resitelu (a loh v nm)
STATUS free_resitele(RESITEL * p_resitel) {

  if (p_resitel != NULL) {
    // nejprve podstruktury
    if ((p_resitel->jmeno_resitele != NULL)){
      free(p_resitel->jmeno_resitele);
    }
    // seznam vech een tohoto eitele
    free_resena_uloha_list(p_resitel);
    // uvoln strukturu
    free(p_resitel);
  }
  return OK;
}

// Alokuj nov seznam eitel
SEZNAM_RESITELU * new_seznam_resitelu(void) {
  SEZNAM_RESITELU * p_seznam_resitelu;          // nov struktura SEZNAM_RESITELU

  // alokuj pam
  p_seznam_resitelu = (SEZNAM_RESITELU *) malloc(sizeof(SEZNAM_RESITELU));
  // inicializan hodnoty
  p_seznam_resitelu->p_first_resitel= NULL;     // jet dn eitel
  p_seznam_resitelu->n_resitelu = 0;
  // pointer na novou alokovanou strukturu
  return p_seznam_resitelu;
}

// Uvoln alokovanou pam seznamu loh
STATUS free_seznam_resitelu(SEZNAM_RESITELU * p_seznam_resitelu) {
  RESITEL * p_resitel;                  // pointery na eitele
  RESITEL * p_next_resitel;             // pointery na dalho eitele

  if (p_seznam_resitelu != NULL) {
    // nejprve podstruktury
    p_resitel = p_seznam_resitelu->p_first_resitel;
    while (p_resitel != NULL) {
      p_next_resitel = p_resitel->p_next_resitel;
      // uvoln strukturu archivu
      free_resitele(p_resitel);
      // move on
      p_resitel = p_next_resitel;
    }
    // uvoln strukturu
    free(p_seznam_resitelu);
  }
  return OK;
}

// Napl strukturu 'datum'
STATUS init_datum(DATUM * p_datum, int den, int mesic, int rok) {
  if (p_datum == NULL) {  // poinetr je NULL
    error_message(DATUM_NOT_ALLOCATED, NULL);
    //D    exit(DATUM_NOT_ALLOCATED);
  }
  else {
    // init the structure
    p_datum->den = den;                    // number of this 'loha' as a string
    p_datum->mesic = mesic;                // name of this 'loha'
    p_datum->rok = rok;                    // number of points for this 'loha'
  }
  return OK;
}

// Alokuj nov een
RESENI * new_reseni(DATUM * p_datum, RESITEL * p_resitel, ULOHA * p_uloha, KOMENTAR komentar, BODY body_reseni) {
  RESENI * p_reseni;                     // nov struktura RESENI

  // alokuj pam
  p_reseni = (RESENI *) malloc(sizeof(RESENI));
  // inicializan hodnoty
  init_datum(&(p_reseni->datum), p_datum->den, p_datum->mesic, p_datum->rok);  // datum tohoto een: den, msc, rok
  p_reseni->p_archiv = p_uloha->p_archiv; // pointer na archiv, ve kterm je een loha
  p_reseni->p_uloha = p_uloha;            // pointer na eenou lohu
  p_reseni->p_resitel = p_resitel;        // pointer na eitele lohy
  p_reseni->komentar = komentar;          // pointer na koment k een
  p_reseni->body_reseni = body_reseni;    // poet bod za een lohy
  p_reseni->radek_souboru = 0;            // slo dku souboru, zatm nen
  p_reseni->p_next_reseni = NULL;         // pointer na dal een
  // pointer na novou alokovanou strukturu
  return p_reseni;
}

// Uvoln pam jednoho een
STATUS free_reseni(RESENI * p_reseni) {
  if (p_reseni != NULL) {
    // nejprve podstruktury
    if (p_reseni->komentar != NULL) {
      free(p_reseni->komentar);
    }
    // uvoln strukturu
    free(p_reseni);
  }
  return OK;
}

// Alokuj nov seznam een
SEZNAM_RESENI * new_seznam_reseni(void) {
  SEZNAM_RESENI * p_seznam_reseni;          // nov struktura SEZNAM_RESENI

  // alokuj pam
  p_seznam_reseni = (SEZNAM_RESENI *) malloc(sizeof(SEZNAM_RESENI));
  // inicializan hodnoty
  p_seznam_reseni->p_first_reseni = NULL; // jet dn een
  p_seznam_reseni->n_reseni = 0;
  // pointer na novou alokovanou strukturu
  return p_seznam_reseni;
}

// Uvoln alokovanou pam seznamu loh
STATUS free_seznam_reseni(SEZNAM_RESENI * p_seznam_reseni) {
  RESENI * p_reseni;                      // pointery na een
  RESENI * p_next_reseni;                 // pointery na dal een

  if (p_seznam_reseni != NULL) {
    // nejprve podstruktury
    p_reseni = p_seznam_reseni->p_first_reseni;
    while (p_reseni != NULL) {
      p_next_reseni = p_reseni->p_next_reseni;
      // uvoln strukturu archivu
      free_reseni(p_reseni);
      // move on
      p_reseni = p_next_reseni;
    }
    // uvoln strukturu
    free(p_seznam_reseni);
  }
  return OK;
}

// Najdi archiv s odpovdajcm jmnem, ppadn vrat NULL
ARCHIV * find_archiv(SEZNAM_ULOH * seznam_uloh, JMENO jmeno_archivu) {
  ARCHIV * p_archiv;                    // pointer na archiv

  // zanme od zatku
  p_archiv = seznam_uloh->p_first_archiv;
  // projdi vechny archivy v seznamu
  while (p_archiv != NULL) {
    // je to hledan archiv?
    if (strcmp(p_archiv->jmeno_archivu, jmeno_archivu) == 0) {
      return p_archiv; // mme ho
    }
    // move on
    p_archiv = p_archiv->p_next_archiv;
  }
  // archiv nebyl nalezen
  return NULL;
}

// Najdi lohu s odpovdajcm jmnem archivu a indexem (stringy), ppadn vrat NULL
ULOHA * find_uloha(SEZNAM_ULOH * seznam_uloh, JMENO jmeno_archivu, CISLO index_ulohy) {
  ARCHIV * p_archiv;                    // pointer na archiv
  ULOHA * p_uloha;                      // pointer na lohu

  // nejprve najdi archiv
  p_archiv = find_archiv(seznam_uloh, jmeno_archivu);
  if (p_archiv == NULL) {
    // kdy nen archiv, nebude ani loha
    return NULL;
  }
  // v archivu zanme od zatku
  p_uloha = p_archiv->p_first_uloha;
  // projdi vechny lohy v archivu
  while (p_uloha != NULL) {
    // je to hledan loha?
    if (p_uloha->index_ulohy == index_ulohy) {
      return p_uloha;  // mme ho!
    }
    // move on
    p_uloha = p_uloha->p_next_uloha;
  }
  // loha nebyla nalezena
  return NULL;
}

// Najdi eitele s danm jmnem, ppadn vrat NULL
RESITEL * find_resitel(SEZNAM_RESITELU * seznam_resitelu, JMENO jmeno_resitele) {
  RESITEL * p_resitel;                    // pointer na resitele

  // zanme od zatku
  p_resitel = seznam_resitelu->p_first_resitel;
  // projdi vechny resitely v seznamu
  while (p_resitel != NULL) {
    // je to hledan resitel?
    if (strcmp(p_resitel->jmeno_resitele, jmeno_resitele) == 0) {
      return p_resitel; // mme ho
    }
    // move on
    p_resitel = p_resitel->p_next_resitel;
  }
  // resitel nebyl nalezen
  return NULL;
}

// Hled pedchoz een lohy
RESENI * find_predchozi_reseni(RESITEL * p_resitel, JMENO jmeno_archivu, CISLO index_ulohy) {
  RESENA_ULOHA * p_resena_uloha;          // pointer pro prochzen seznamu eench loh jednoho eitele

  // projdi seznam eitele od zatku
  p_resena_uloha = p_resitel->p_first_resena_uloha;
  while (p_resena_uloha != NULL) {
    if ((strcmp(p_resena_uloha->p_reseni->p_archiv->jmeno_archivu, jmeno_archivu) == 0) && (p_resena_uloha->p_reseni->p_uloha->index_ulohy == index_ulohy)) {
      return(p_resena_uloha->p_reseni);
    }
    p_resena_uloha = p_resena_uloha->p_next_resena_uloha;
  }
  // nebyla nalezena
  return NULL;
}

// Porovnej dv lohy podle sla lohy; vrt integer<0 jestlie archiv1.index1 < archiv2.index2
int compare_ulohy(ULOHA * p_uloha1, ULOHA * p_uloha2) {
  // jsou rzn archivy?
  if (p_uloha1->p_archiv->cislo_archivu != p_uloha2->p_archiv->cislo_archivu) {
    return(p_uloha1->p_archiv->cislo_archivu - p_uloha2->p_archiv->cislo_archivu);  // rzn archivy
  }
  else {  // stejn archiv
    return(p_uloha1->index_ulohy - p_uloha2->index_ulohy);  // rzn indexy
  }
}

// Extrahuje text mezi '{' '}'
char * one_argument(char ** radek) {
  char * pozice = * radek;
  char * polozka_begin = NULL;
  char * polozka_end = NULL;
  char * polozka = NULL;

  // pesko vechny znaky a po '{'
  while ((* pozice != '\0') && (* pozice != '{')) {
    pozice++;  // move on
  }
  // pedasn konec?
  if (* pozice == '\0') {
    error_message(NOT_COMPLETE_DATA, * radek);
    //D    exit(NOT_COMPLETE_DATA);
  }
  polozka_begin = ++pozice;  // pesko vetn '{'
  // pesko vechny znaky a po '}' (bez '}')
  while ((* pozice != '\0') && (* pozice != '}')) {
    pozice++;  // move on
  }
  // pedasn konec?
  if (* pozice == '\0') {
    error_message(NOT_COMPLETE_DATA, * radek);
    //D    exit(NOT_COMPLETE_DATA);
  }
  polozka_end = pozice;      // nepeskakovat '}'
  // nen skryt backspace?
  //D  if (polozka_end <= polozka_begin) {
  //D    error_message(NOT_COMPLETE_DATA, * radek);
  //D  }
  // alokuj pam pro text poloky
  polozka = (char *) malloc(sizeof(char)*((polozka_end-polozka_begin)+1));  // vetn '\0'
  strncpy(polozka, polozka_begin, (polozka_end-polozka_begin));             // neukoneno
  polozka[(polozka_end-polozka_begin)] = '\0';                              // ukoni
  // zm argument
  * radek = ++pozice;        // pesko '}'
  // hotovo
  return(polozka);
}

// Vrt jmno archivu jako string v nov alokovan pamti
JMENO archive_name(char * text) {
  char jmeno_archivu_tmp[MAX_CISLO_ARCHIVU+1];  // jmno archivu (pomocn promnn)
  JMENO jmeno_archivu;                          // jmno archivu (alokovan pam)
  int i;                                        // indexov promnn

  // jmno kon '.' nebo '\0'
  i = 0;
  while ((text[i] != '.') && (text[i] != '\0') && (i < MAX_CISLO_ARCHIVU)) {
    jmeno_archivu_tmp[i] = text[i++];
  }
  jmeno_archivu_tmp[i] = '\0';
  // pozor, dn jmno archivu nen
  if (i == 0) {
    error_message(NO_ARCHIVE_NAME, text);
    //D    exit(NO_ARCHIVE_NAME);
  }
  // alokuj novou pam
  jmeno_archivu = (char *) malloc(sizeof(char)*(strlen(jmeno_archivu_tmp)+1));
  strcpy(jmeno_archivu, jmeno_archivu_tmp);
  return jmeno_archivu;
}

// Vrt index archivu jako string v nov alokovan pamti
JMENO uloha_index(char * text) {
  char id_ulohy_tmp[MAX_INDEX_ULOHY+1];  // index lohy (pomocn promnn)
  JMENO id_ulohy;                        // index lohy (alokovan pam)
  int i;                                 // indexov promnn

  // pesko po prvn '.'
  while ((text[0] != '.') && (text[0] != '\0')) {
    text++;
  }
  // vetn prvn '.'
  if (text[0] == '.') {
    text++;
  }
  // jmno kon '.' nebo '\0'
  i = 0;
  while ((text[i] != '.') && (text[i] != '\0') && (i < MAX_INDEX_ULOHY)) {
    id_ulohy_tmp[i] = text[i++];
  }
  // kon ID tekou?
  //T  if (text[i] == '\0') {
  //T    error_message(INCOMPLETE_ID, text);
  //T  }
  id_ulohy_tmp[i] = '\0';
  // pozor, dn index lohy nen
  if (i == 0) {
    error_message(NO_ULOHA_INDEX, text);
    //D    exit(NO_ULOHA_INDEX);
  }
  // alokuj novou pam
  id_ulohy = (char *) malloc(sizeof(char)*(strlen(id_ulohy_tmp)+1));
  strcpy(id_ulohy, id_ulohy_tmp);
  // vra pinter na nov alokovanou strukturu
  return id_ulohy;
}

// Pidej archiv do seznamu loh
STATUS add_archiv(SEZNAM_ULOH * p_seznam_uloh, ARCHIV * p_archiv) {
  ARCHIV * p_last_archiv;                // pointer pro prochzen seznamu archiv

  // nen nhodou archiv v seznamu podruh?
  if (find_archiv(p_seznam_uloh, p_archiv->jmeno_archivu) != NULL) {
    fprintf(stderr, "Archiv s slem: %s. je v seznamu dvakrt!\n", p_archiv->jmeno_archivu);
    error_message(ARCHIV_TWICE, p_archiv->jmeno_archivu);
    //D    exit(ARCHIV_TWICE);
  }
  // je to prvn archiv?
  if (p_seznam_uloh->p_first_archiv == NULL) {
    p_seznam_uloh->p_first_archiv = p_archiv;
  }
  // nen, zaa na konec
  else {
    p_last_archiv = p_seznam_uloh->p_first_archiv;
    while (p_last_archiv->p_next_archiv != NULL) {
      p_last_archiv = p_last_archiv->p_next_archiv;
    }
    p_last_archiv->p_next_archiv = p_archiv;
  }
  // zapotej dal archiv
  p_seznam_uloh->n_archivu += 1;
  // done
  return OK;
}

// Pidej lohu do seznamu loh
STATUS add_uloha(SEZNAM_ULOH * p_seznam_uloh, ULOHA * p_uloha) {
  ULOHA * p_last_uloha;                  // pointer pro prochzen seznamu loh

  // nen nhodou loha v seznamu podruh?
  if (find_uloha(p_seznam_uloh, p_uloha->p_archiv->jmeno_archivu, p_uloha->index_ulohy) != NULL) {
    fprintf(stderr, "loha s slem: %s.%d. je v archivu dvakrt!\n", p_uloha->p_archiv->jmeno_archivu, p_uloha->index_ulohy);
    error_message(ULOHA_TWICE, p_uloha->jmeno_ulohy);
    //D    exit(ULOHA_TWICE);
  }
  // zaa lohu do struktury
  if (p_uloha->p_archiv->p_first_uloha == NULL) {  // je to prvn loha?
    p_uloha->p_archiv->p_first_uloha = p_uloha;
  }
  // nen prvn, m zatm nejmen slo?
  else if (compare_ulohy(p_uloha, p_uloha->p_archiv->p_first_uloha) < 0) {
    p_uloha->p_next_uloha = p_uloha->p_archiv->p_first_uloha;
    p_uloha->p_archiv->p_first_uloha = p_uloha;
  }
  else {  // zaa na SV MSTO podle sel
    p_last_uloha = p_uloha->p_archiv->p_first_uloha;
    while (p_last_uloha->p_next_uloha != NULL) {
      if (compare_ulohy(p_uloha, p_last_uloha->p_next_uloha) < 0) {  // a na msto, kdy m dal loha vy index
	break;
      }
      p_last_uloha = p_last_uloha->p_next_uloha;
    }
    // zaa do struktury
    p_uloha->p_next_uloha = p_last_uloha->p_next_uloha;
    p_last_uloha->p_next_uloha = p_uloha;
  }
  // zapotej dal lohu
  p_seznam_uloh->n_uloh += 1;
  // done
  return OK;
}

// Pidej eitele do seznamu eitel
STATUS add_resitel(SEZNAM_RESITELU * p_seznam_resitelu, RESITEL * p_resitel) {
  RESITEL * p_last_resitel;              // pointer pro prochzen seznamu eitel

  // nen nhodou eitel v seznamu podruh?
  if (find_resitel(p_seznam_resitelu, p_resitel->jmeno_resitele) != NULL) {
    fprintf(stderr, "eitel '%s' je v seznamu dvakrt!\n", p_resitel->jmeno_resitele);
    error_message(RESITEL_TWICE, p_resitel->jmeno_resitele);
    //D    exit(RESITEL_TWICE);
  }
  /*?
  // zaa eitele do struktury eitel na zatek (a nemusme dlouho hledat)
  p_resitel->p_next_resitel = p_seznam_resitelu->p_first_resitel;
  p_seznam_resitelu->p_first_resitel = p_resitel;
  */
  // je to prvn eitel?
  if (p_seznam_resitelu->p_first_resitel == NULL) {
    p_seznam_resitelu->p_first_resitel = p_resitel;
  }
  // nen, zaa na konec (LZE ZRYCHLIT NA ZATEK, ale chceme zachovat poad stejn obodovanch podle data)
  else {
    p_last_resitel = p_seznam_resitelu->p_first_resitel;
    while (p_last_resitel->p_next_resitel != NULL) {
      p_last_resitel = p_last_resitel->p_next_resitel;
    }
    p_last_resitel->p_next_resitel = p_resitel;
  }
  // zapotej dalho eitele
  p_seznam_resitelu->n_resitelu++;
  // done
  return OK;
}

// Pidej een do seznamu een
STATUS add_reseni(SEZNAM_RESENI * p_seznam_reseni, RESENI * p_reseni) {
  // zaa een do struktury seznamu een na zatek (a nemusme dlouho hledat)
  p_reseni->p_next_reseni = p_seznam_reseni->p_first_reseni;
  p_seznam_reseni->p_first_reseni = p_reseni;
  // zapotej dal een
  p_seznam_reseni->n_reseni++;
  // done
  return OK;
}

// Pidej een lohy do seznamu een uloh konkrtnho eitele
STATUS add_resena_uloha(RESITEL * p_resitel, RESENA_ULOHA * p_resena_uloha) {
  RESENA_ULOHA * p_last_resena_uloha;          // pointer pro prochzen seznamu eench loh jednoho eitele

  // nejprve najt, jestli nen podruh
  if (find_predchozi_reseni(p_resitel, p_resena_uloha->p_reseni->p_archiv->jmeno_archivu, p_resena_uloha->p_reseni->p_uloha->index_ulohy) != NULL) {
    error_message(VYRESENA_ULOHA_TWICE, p_resena_uloha->p_reseni->p_uloha->jmeno_ulohy);
    //D    exit(VYRESENA_ULOHA_TWICE);
  }
  // zaadit een mezi ostatn een
  if (p_resitel->p_first_resena_uloha == NULL) {  // je to jeho prvn een?
    // toto je jeho prvn een
    p_resitel->p_first_resena_uloha = p_resena_uloha;
  }
  else {
    // m zatm nejmen slo?
    if (compare_ulohy(p_resena_uloha->p_reseni->p_uloha, p_resitel->p_first_resena_uloha->p_reseni->p_uloha) < 0) {
      // ano, zaa na zatek
      p_resena_uloha->p_next_resena_uloha = p_resitel->p_first_resena_uloha;
      p_resitel->p_first_resena_uloha = p_resena_uloha;
    }
    else {  // zaa na SV MSTO
      p_last_resena_uloha = p_resitel->p_first_resena_uloha;
      while (p_last_resena_uloha->p_next_resena_uloha != NULL) {
	// a na msto, kdy m dal loha vy index
	if (compare_ulohy(p_resena_uloha->p_reseni->p_uloha, p_last_resena_uloha->p_next_resena_uloha->p_reseni->p_uloha) < 0) {
	  break;
	}
        p_last_resena_uloha = p_last_resena_uloha->p_next_resena_uloha;
      }
      // zaa
      p_resena_uloha->p_next_resena_uloha = p_last_resena_uloha->p_next_resena_uloha;
      p_last_resena_uloha->p_next_resena_uloha = p_resena_uloha;
    }
  }
  // zapotej dal een
  p_resitel->pocet_uloh++;
  if (p_resena_uloha->p_reseni->body_reseni == p_resena_uloha->p_reseni->p_uloha->body_ulohy) {
    p_resitel->pocet_plnych_uloh++;
  }
  // zapotej body za een
  p_resitel->suma_bodu += p_resena_uloha->p_reseni->body_reseni;
  // done
  return OK;
}

// Pidej een lohy do seznamu een konkrtn lohy
STATUS add_reseni_ulohy(ULOHA * p_uloha, RESENI_ULOHY * p_reseni_ulohy) {
  // je to prvn een tto lohy?
  if (p_uloha->p_first_reseni_ulohy == NULL) {
    // toto je prvn een
    p_uloha->p_first_reseni_ulohy = p_reseni_ulohy;
  }
  else {  // zaa na zatek seznamu (a nemusme dlouho hledat)
    p_reseni_ulohy->p_next_reseni_ulohy = p_uloha->p_first_reseni_ulohy;
    p_uloha->p_first_reseni_ulohy = p_reseni_ulohy;
  }
  // zapotej dal een lohy
  p_uloha->n_reseni++;
  if (p_uloha->body_ulohy == p_reseni_ulohy->p_reseni->body_reseni) {
    p_uloha->n_uplnych_reseni++;
  }
  // done
  return OK;
}

// Nate jednu lohu a vrt pointer na ni
// Syntaxe dku je '{slo lohy}{jmno lohy}{body}'
ULOHA * nacti_ulohu(PARAMETERS * p_parameters, SEZNAM_ULOH * seznam_uloh, char * radek) {
  ARCHIV * p_archiv;                     // nov struktura archivu (alokuj v ppad, jestli ve nateno v podku)
  ULOHA * p_uloha;                       // nov struktura lohy (alokuj v ppad, jestli ve nateno v podku)
  char * id_text;                        // slo archvu a lohy jako string
  TYP typ_jmena_archivu = TYP_CISLA_NULL;  // jak je uloeno jmno archivu?
  JMENO jmeno_archivu = NULL;            // jmno archivu jako string
  CISLO cislo_archivu = 0;               // jmno archivu jako slo
  JMENO id_ulohy = NULL;                 // slo lohy jako string
  CISLO index_ulohy = 0;                 // slo (index) lohy jako slo
  JMENO_ULOHY jmeno_ulohy = NULL;        // jmno lohy
  char * body_text = NULL;               // body za lohu jako string
  BODY body_ulohy;                       // body za lohu

  // nati slo lohy
  if ((id_text = one_argument(&radek)) == NULL) {
    error_message(NO_NUMBER_GIVEN, radek);
    //D    exit(NO_NUMBER_GIVEN);
  }

  // jmno/slo archivu
  jmeno_archivu = archive_name(id_text);
  // je jmno archvu slo?
  if (sscanf(jmeno_archivu, "%d", &cislo_archivu) == 1) {
    // je to slo!
    typ_jmena_archivu = TYP_CISLA_NUM;
  }
  else {
    // jenom string
    typ_jmena_archivu = TYP_CISLA_STR;
    cislo_archivu = MAX_ARCHIV;
  }

  // jmno/slo lohy
  id_ulohy = uloha_index(id_text);
  // je index lohy slo?
  if (sscanf(id_ulohy, "%d", &index_ulohy) != 1) {
    // nen slo!
    error_message(ULOHA_INDEX_NOT_NUMBER, id_ulohy);
    //D    exit(ULOHA_INDEX_NOT_NUMBER);
  }
  // uvoln pam pro id_text (u se nepouije)
  free(id_text);
  // uvoln pam pro id_ulohu (u se nepouije)
  free(id_ulohy);

  // nati jmno lohy
  if ((jmeno_ulohy = one_argument(&radek)) == NULL) {
    error_message(NO_ULOHA_NAME_GIVEN, radek);
    //D    exit(NO_ULOHA_NAME_GIVEN);
  }

  // nati body za lohu
  if ((body_text = one_argument(&radek)) == NULL) {
    error_message(NO_POINTS_GIVEN, radek);
    //D    exit(NO_POINTS_GIVEN);
  }
  // body jako slo
  if (sscanf(body_text, "%u", &body_ulohy) != 1) {
    error_message(POINTS_ARE_NOT_INTEGER, radek);
    //D    exit(POINTS_ARE_NOT_INTEGER);
  }
  // uvoln pam pro body_text (u se nepouije)
  free(body_text);

  // najdi odpovdajc archiv
  p_archiv = find_archiv(seznam_uloh, jmeno_archivu);
  // neexistuje, tak ho zalo a zaa do seznamu
  if (p_archiv == NULL) {
    p_archiv = new_archiv(typ_jmena_archivu, jmeno_archivu, cislo_archivu);
    // pidej archiv do seznamu loh
    add_archiv(seznam_uloh, p_archiv);
  }
  // nov struktura lohy (ihned zaazen i link na archiv)
  p_uloha = new_uloha(p_archiv, index_ulohy, jmeno_ulohy, body_ulohy);

  // pro kontrolu
  if (p_parameters->info_archiv >= 3) {
  //T  if (p_parameters->info_archiv >= 1) {
    printf("natm lohu %s.%d. '%s' (%db)\n", p_archiv->jmeno_archivu, p_uloha->index_ulohy, p_uloha->jmeno_ulohy, p_uloha->body_ulohy);
  }

  // uvoln pam pro jmeno_archivu (u se nepouije)
  free(jmeno_archivu);
  // return the new structure
  return(p_uloha);
}

// Nate soubor se seznamem loh a rozdl je do archiv
// Syntax souboru: {slo lohy}{jmno lohy}{body}
// Komente jsou uvozeny znaky '%', '#' nebo '/'
SEZNAM_ULOH * read_ulohy(PARAMETERS * p_parameters) {
  SEZNAM_ULOH * seznam_uloh;             // seznam vech loh ze souboru
  ULOHA * uloha;                         // naten loha
  FILE * soubor_ulohy;                   // soubor se seznamem loh
  char radek[MAX_RADEK];                 // pomocn string pro vstup dku
  char * sub_radek;                      // pozice ve stringu 'radek'
  CISLO_RADKU cislo_radku = 0;           // poet dk v souboru vetn koment

  // Alokuj pam
  seznam_uloh = new_seznam_uloh();

  // Vypi zprvu co se dj
  if (p_parameters->verbosity_level > 2) {
    fprintf(stdout, "Otevrm soubor '%s' ...\n", p_parameters->in_file_ulohy);
  }
  // open 'lohy' file
  if ((soubor_ulohy = fopen(p_parameters->in_file_ulohy, "rt")) == NULL) {
    error_message(CANT_OPEN_FILE_ULOHY, p_parameters->in_file_ulohy);
    //D    exit(CANT_OPEN_FILE_ULOHY);
  }
  // zpracuj soubor
  while (fgets(radek, MAX_RADEK, soubor_ulohy) != NULL) {
    // nate jeden dek ze souboru, je-li pli dlouh (co nem bt) nape info
    if (radek[strlen(radek)-1] == '\n') {    // mme cel dal dek?
      cislo_radku++;
    }
    else {     // nebo byl pli dlouh?
      error_message(LINE_ULOHY_TOO_LONG, radek);
      //D      exit(LINE_ULOHY_TOO_LONG);
    }
    // pesko whitespaces
    sub_radek = radek;
    while (isspace(* sub_radek)) {
      sub_radek++;
    }
    // pesko skoro przdn dky (minimln "{}{}{}")
    if ((strlen(sub_radek)) <= VERY_SHORT_LINE_ULOHY) {
      continue;
    }
    // pesko dky s komentem (zanaj '%', '#' nebo '/')
    if ((* sub_radek == '%') || (* sub_radek == '#') || (* sub_radek == '/')) {
      continue;
    }
    // nati lohu ze zbytku dku
    uloha = nacti_ulohu(p_parameters, seznam_uloh, sub_radek);
    // dek soubory
    uloha->radek_souboru = cislo_radku;
    // zaa lohu do struktury seznamu een
    add_uloha(seznam_uloh, uloha);
    // mme dal lohu
    //D    seznam_uloh->n_uloh++;
  }
  // Zavi soubor s lohami
  if (fclose(soubor_ulohy) != 0) {
    error_message(NOT_CLOSED_FILE_ULOHY, p_parameters->in_file_ulohy);
    //D    exit(NOT_CLOSED_FILE_ULOHY);
  }
  // Byla vbec njak loha?
  if (seznam_uloh->n_uloh == 0) {
    error_message(NO_ULOHY_IN_FILE_ULOHY, p_parameters->in_file_ulohy);
  }
  if (p_parameters->verbosity_level > 2) {
    fprintf(stdout, "... zaveno. (nateno %d loh z %lu dk)\n", seznam_uloh->n_uloh, cislo_radku);
  }
  // nov struktura
  return seznam_uloh;
}

// Vype seznam loh
void print_seznam_uloh(SEZNAM_ULOH * seznam_uloh) {
  ARCHIV * p_archiv;                // pointer pro prochzen seznamu archiv
  ULOHA * p_uloha;                  // pointer pro prochzen seznamu loh

  // Vype neprzdn seznam
  if (seznam_uloh != NULL) {
    printf("Seznam loh:\n");
    // od prvnho
    p_archiv = seznam_uloh->p_first_archiv;
    while (p_archiv!= NULL) {
      printf("Archiv %s (%d) obsahuje %d loh:\n", p_archiv->jmeno_archivu, p_archiv->cislo_archivu, p_archiv->n_uloh_v_archivu);
      // Vype vechny lohy v archivu
      p_uloha = p_archiv->p_first_uloha;
      while (p_uloha != NULL) {
        printf("  loha %s.%d (%d.%d.) %s (%db) / .%lu\n", p_uloha->p_archiv->jmeno_archivu, p_uloha->index_ulohy, p_uloha->p_archiv->cislo_archivu, p_uloha->index_ulohy, p_uloha->jmeno_ulohy, p_uloha->body_ulohy, p_uloha->radek_souboru);
	// move on
	p_uloha = p_uloha->p_next_uloha;
      }
      // move on
      p_archiv = p_archiv->p_next_archiv;
    }
  }
  return;
}

// Vype seznam een
void print_seznam_reseni(SEZNAM_RESENI * seznam_reseni) {
  RESENI * p_reseni;                // pointer pro prochzen seznamu een

  // Vype neprzdn seznam
  if (seznam_reseni != NULL) {
    printf("Seznam een:\n");
    // od prvnho
    p_reseni = seznam_reseni->p_first_reseni;
    while (p_reseni!= NULL) {
      printf("%2d.%2d.%d vyeil '%s' lohu '%s.%d. %s' (%db/%db)", p_reseni->datum.den, p_reseni->datum.mesic, p_reseni->datum.rok, p_reseni->p_resitel->jmeno_resitele, p_reseni->p_uloha->p_archiv->jmeno_archivu, p_reseni->p_uloha->index_ulohy, p_reseni->p_uloha->jmeno_ulohy, p_reseni->body_reseni, p_reseni->p_uloha->body_ulohy);
      if (p_reseni->komentar != NULL) {
        printf("; koment '%s'", p_reseni->komentar);
      }
      printf(" / .%lu\n", p_reseni->radek_souboru);
      // move on
      p_reseni = p_reseni->p_next_reseni;
    }
  }
  return;
}

// Vype seznam eitel
void print_seznam_resitelu(SEZNAM_RESITELU * seznam_resitelu) {
  RESITEL * p_resitel;                    // pointer pro prochzen seznamu eitel
  RESENA_ULOHA * p_resena_uloha;          // pointer pro prochzen seznamu eench loh

  // Vype neprzdn seznam
  if (seznam_resitelu != NULL) {
    printf("Seznam eitel:\n");
    // od prvnho
    p_resitel = seznam_resitelu->p_first_resitel;

    while (p_resitel!= NULL) {
      printf("eitel '%s' m %u bod; vyeil %ld loh (%ld z nich za pln poet)\n", p_resitel->jmeno_resitele, p_resitel->suma_bodu, p_resitel->pocet_uloh, p_resitel->pocet_plnych_uloh);
      // Vype vechny lohy vyeen eitelem
      p_resena_uloha = p_resitel->p_first_resena_uloha;
      while (p_resena_uloha != NULL) {
        printf("  %2d.%2d.%d: loha '%s.%d. %s' (%db/%db)", p_resena_uloha->p_reseni->datum.den, p_resena_uloha->p_reseni->datum.mesic, p_resena_uloha->p_reseni->datum.rok, p_resena_uloha->p_reseni->p_uloha->p_archiv->jmeno_archivu, p_resena_uloha->p_reseni->p_uloha->index_ulohy, p_resena_uloha->p_reseni->p_uloha->jmeno_ulohy, p_resena_uloha->p_reseni->body_reseni, p_resena_uloha->p_reseni->p_uloha->body_ulohy);
        if (p_resena_uloha->p_reseni->komentar != NULL) {
          printf("; koment '%s'\n", p_resena_uloha->p_reseni->komentar);
        }
        else {
          printf("\n");
        }
	// move on
	p_resena_uloha = p_resena_uloha->p_next_resena_uloha;
      }
      // move on
      p_resitel = p_resitel->p_next_resitel;
    }
  }
  return;
}

// Uspod lohy v seznamu podle sel
void usporadej_ulohy(SEZNAM_ULOH * p_seznam_uloh) {
  //?  RESENA_ULOHA * p_first_resena_uloha;   // pointer na prvn vyeenou lohu

  // zatm nen poteba
  // ??

  return;
}

// Uspod lohy kadho eitele podle sel
void counting_sort_resene_ulohy(SEZNAM_RESITELU * p_seznam_resitelu) {
  //T  RESENA_ULOHA ** p_original_list;          // originln seznam loh eitele
  //T  RESENA_ULOHA ** p_sorted_list;            // uspodan seznam loh eitele
  RESITEL * p_resitel;                      // pointer pro prochzen seznamu eitel
  //D?  RESENA_ULOHA * p_first_resena_uloha;   // pointer na prvn vyeenou lohu


  // je co uspodvat?
  if ((p_seznam_resitelu == NULL) || (p_seznam_resitelu->p_first_resitel == NULL) || (p_seznam_resitelu->n_resitelu == 0)) {
    // nen co uspodvat
    return;
  }
  // pro kadho eitele
  p_resitel = p_seznam_resitelu->p_first_resitel;
  while (p_resitel != NULL) {

    // !!

    // move on
    p_resitel = p_resitel->p_next_resitel;
  }
  // done
  return;
}

// Uspod eitele v seznamu podle abecedy

// Uspod eitele v seznamu podle potu bod
STATUS counting_sort_resitele_by_points(SEZNAM_RESITELU * p_seznam_resitelu) {
  RESITEL ** p_original_list;               // originln seznam
  RESITEL ** p_sorted_list;                 // uspodan seznam
  BODY max_suma;                            // maximum soutu bod
  RESITEL * p_resitel;                      // 'eitel' v originlnm seznamu
  int i, j;                                 // index variables
  int * count;                              // pole s poty sout

  // Je vbec co uspodvat? (mme vce ne dva eitele?)
  if ((p_seznam_resitelu == NULL) || (p_seznam_resitelu->p_first_resitel == NULL) || (p_seznam_resitelu->p_first_resitel->p_next_resitel == NULL)) {
    // already sorted ;o)
    return OK;
  }

  // alokuj pam pro pole pointer na eitele (originln seznam a uspodan seznam)
  p_original_list = (RESITEL **) malloc(p_seznam_resitelu->n_resitelu * sizeof(RESITEL *));
  p_sorted_list = (RESITEL **) malloc(p_seznam_resitelu->n_resitelu * sizeof(RESITEL *));

  // napl pole originlnm seznamem
  max_suma = 0;
  i = 0;
  p_resitel = p_seznam_resitelu->p_first_resitel;
  while (p_resitel != NULL) {
    // inicializuj originln pole
    p_original_list[i++] = p_resitel;
    // jak je maximum soutu bod?
    if (p_resitel->suma_bodu > max_suma) {
      max_suma = p_resitel->suma_bodu;
    }
    // move on
    p_resitel = p_resitel->p_next_resitel;
  }
  // rychl kontrrola potu eitel
  if (i != p_seznam_resitelu->n_resitelu) {
    error_message(ERROR_IN_RESITEL_STRUCTURE, NULL);
    //D    exit(ERROR_IN_RESITEL_STRUCTURE);
  }
  // alokuj pam pro pole potu sout
  count = (int *) malloc((max_suma+1) * sizeof(int));
  // inicializuj pole potu sout
  for (i=0; i <= max_suma; i++) {
    count[i] = 0;
  }
  // jak jsou souty?
  for (j=0; j < p_seznam_resitelu->n_resitelu; j++) {
    count[p_original_list[j]->suma_bodu]++;
  }
  // poty mench nebo stejnch sout
  for (i=max_suma-1; i >= 0; i--) {
    count[i] += count[i+1];
  }
  // vytvo setzen seznam
  for (j=p_seznam_resitelu->n_resitelu-1; j >= 0; j--) {
    p_sorted_list[(count[p_original_list[j]->suma_bodu]--)-1] = p_original_list[j];
  }
  // do seznamu eitel pekopruj uspodan seznam
  if (p_seznam_resitelu->p_first_resitel != NULL) {
    p_seznam_resitelu->p_first_resitel = p_sorted_list[0];
    p_resitel = p_seznam_resitelu->p_first_resitel;
    for (j=1; j < p_seznam_resitelu->n_resitelu; j++) {
      p_resitel->p_next_resitel = p_sorted_list[j];
      // move on
      p_resitel = p_resitel->p_next_resitel;
    }
    // za poslednm eitelem nic nen
    p_resitel->p_next_resitel = NULL;
  }
  // uvoln alokovanou pam
  free(p_original_list);
  free(p_sorted_list);
  free(count);
  // seazeno
  return OK;
}

// Uspod eitele v seznamu podle potu loh

// ??


// Nate jedno een a vrt pointer na nj
// Syntaxe dku je: '{datum}{jmno eitele}{slo lohy}{body}'
RESENI * nacti_reseni(PARAMETERS * p_parameters, SEZNAM_ULOH * p_seznam_uloh, SEZNAM_RESITELU * p_seznam_resitelu, char * radek) {
  RESENI * p_reseni;                     // nov struktura een (alokuj v ppad, jestli ve nateno v podku)
  RESITEL * p_resitel;                   // pointer na strukturu eitele
  ULOHA * p_uloha;                       // pointer na strukturu lohy
  char * datum_text = NULL;              // datum een jako string
  DATUM datum;                           // datum een jako struktura
  static DATUM predchozi_datum;          // datum pedchozho een jako struktura
  JMENO jmeno_resitele;                  // jmno eitele
  char * id_text;                        // slo archvu a lohy jako string
  TYP typ_jmena_archivu = TYP_CISLA_NULL;  // jak je uloeno jmno archivu?
  JMENO jmeno_archivu = NULL;            // jmno archivu jako string
  CISLO cislo_archivu = 0;               // jmno archivu jako slo
  JMENO id_ulohy = NULL;                 // slo lohy jako string
  CISLO index_ulohy = 0;                 // slo (index) lohy jako slo
  char komentar_text[MAX_KOMENTAR];      // koment k tomuto een jako text
  KOMENTAR komentar;                     // pointer na alokovanou pam pro koment
  char * body_text = NULL;               // body za een lohy jako string
  BODY body_reseni;                      // body za een lohy jako int
  char * text_varovani[MAX_RADEK];       // text varovn

  // nati datum een
  if ((datum_text = one_argument(&radek)) == NULL) {
    error_message(NO_DATE_GIVEN, radek);
    //D    exit(NO_DATE_GIVEN);
  }
  if (sscanf(datum_text, "%d.%d.%d", &(datum.den), &(datum.mesic), &(datum.rok)) != 3) {
    error_message(WRONG_DATE_FORMAT, datum_text);
    //D    exit(WRONG_DATE_FORMAT);
  }
  // uvoln pam pro datum_text (u se nepouije)
  free(datum_text);

  // kontrola, zda jsou een v souboru seazena chronologicky
  if (p_seznam_resitelu->n_resitelu >= 1) {  // a druhho
    if ((datum.rok < predchozi_datum.rok) ||
       ((datum.rok == predchozi_datum.rok) && (datum.mesic < predchozi_datum.mesic)) ||
       ((datum.rok == predchozi_datum.rok) && (datum.mesic == predchozi_datum.mesic) && (datum.den < predchozi_datum.den))) {
        // jen varovn
        sprintf(text_varovani, "{%d.%d.%d}%sPedchoz een: %d.%d.%d", datum.den, datum.mesic, datum.rok, radek, predchozi_datum.den, predchozi_datum.mesic, predchozi_datum.rok);
        warning_message(RESENI_NOT_CHRONOLOGIC, text_varovani);
    }
  }
  // zapamatuj si
  predchozi_datum.rok = datum.rok;
  predchozi_datum.mesic = datum.mesic;
  predchozi_datum.den = datum.den;

  // nati jmno eitele
  if ((jmeno_resitele = one_argument(&radek)) == NULL) {
    error_message(NO_RESITEL_NAME_GIVEN, radek);
    //D    exit(NO_RESITEL_NAME_GIVEN);
  }
  // Najdi eitele, ppadn zalo novho
  if ((p_resitel = find_resitel(p_seznam_resitelu, jmeno_resitele)) == NULL) {
    p_resitel = new_resitel(jmeno_resitele);
    // pidej do seznamu
    add_resitel(p_seznam_resitelu, p_resitel);
  }
  // uvoln pam pro jmeno_resitele (u se nepouije)
  free(jmeno_resitele);

  // nati slo lohy
  if ((id_text = one_argument(&radek)) == NULL) {
    error_message(NO_NUMBER_GIVEN, radek);
    //D    exit(NO_NUMBER_GIVEN);
  }

  // jmno/slo archivu
  jmeno_archivu = archive_name(id_text);
  // je jmno archvu slo?
  if (sscanf(jmeno_archivu, "%d", &cislo_archivu) == 1) {
    // je to slo!
    typ_jmena_archivu = TYP_CISLA_NUM;
  }
  else {
    // jenom string
    typ_jmena_archivu = TYP_CISLA_STR;
    cislo_archivu = MAX_ARCHIV;
  }

  // jmno/slo lohy
  id_ulohy = uloha_index(id_text);
  // je index lohy slo?
  if (sscanf(id_ulohy, "%d", &index_ulohy) != 1) {
    // nen slo!
    error_message(ULOHA_INDEX_NOT_NUMBER, id_ulohy);
    //D    exit(ULOHA_INDEX_NOT_NUMBER);
  }

  // kon ID tekou? (celkem 2 teky)
  if (strlen(id_text) < strlen(jmeno_archivu)+strlen(id_ulohy)+2) {
    error_message(INCOMPLETE_ID, id_text);
  }
  // Koment
  sprintf(komentar_text, "%s", (id_text+strlen(jmeno_archivu)+1+strlen(id_ulohy)+1));
  // Je vbec njak koment?
  if (strlen(komentar_text) > 0) {  // ano, koment je nenulov dlky
    komentar = (KOMENTAR) malloc(sizeof(char)*(strlen(komentar_text)+1));
    sprintf(komentar, komentar_text);
  }
  else {  // bez komente
    komentar = NULL;
  }
  // uvoln pam pro id_text (u se nepouije)
  free(id_text);
  // uvoln pam pro id_ulohu (u se nepouije)
  free(id_ulohy);

  // najdi odpovdajc archiv a lohu
  if ((p_uloha = find_uloha(p_seznam_uloh, jmeno_archivu, index_ulohy)) == NULL) {
    // 'loha' nen v archivu
    error_message(ULOHA_NOT_IN_ARCHIVE, id_text);
    //D    exit(ULOHA_NOT_IN_ARCHIVE);
  }

  // nati body za een
  if ((body_text = one_argument(&radek)) == NULL) {
    error_message(NO_POINTS_GIVEN, radek);
    //D    exit(NO_POINTS_GIVEN);
  }
  // Kolik bod dostaneme?
  if (sscanf(body_text, "%u", &body_reseni) == 1) {
    // Nen to pli? (vc ne se za lohu me dostat)
    if (body_reseni > p_uloha->body_ulohy) {
      // chyba! bod je pli!
      if (p_parameters->info_resitele >= 1) {
        fprintf(stdout, "'%s' m za lohu '%s' %db msto %db (vce ne 100%%)!\n", p_resitel->jmeno_resitele, p_uloha->jmeno_ulohy, body_reseni, p_uloha->body_ulohy);
      }
      printf("\nBODY: %db\n", p_uloha->body_ulohy);
      error_message(MORE_POINTS_THAN_TOTAL, body_text);
      //D      exit(MORE_POINTS_THAN_TOTAL);
    } 
    else {
      // vypi info, jestli mn bod ne celkov souet
      if ((p_parameters->info_resitele >= 3) && (body_reseni < p_uloha->body_ulohy)) {
        fprintf(stdout, "'%s' m za lohu '%s' %db msto %db.\n", p_resitel->jmeno_resitele, p_uloha->jmeno_ulohy, body_reseni, p_uloha->body_ulohy);
      }
    }
  }
  else {  // je-li znak '*' nebo jakkoli jin...
    // mme pln poet bod?
    if (strcmp(body_text, "*") == 0) {
      body_reseni = p_uloha->body_ulohy;
    }
    else {
      error_message(POINTS_ARE_NOT_INTEGER, body_text);
      //D      exit(POINTS_ARE_NOT_INTEGER);
    }
  }
  // uvoln pam pro body_text (u se nepouije)
  free(body_text);

  // nov struktura lohy (ihned zaazen i link na archiv a lohu)
  p_reseni = new_reseni(&datum, p_resitel, p_uloha, komentar, body_reseni);

  // pro kontrolu
  if (p_parameters->info_resitele >= 3) {
  //T  if (p_parameters->info_resitele >= 1) {
    printf("natm een: dne %d.%d.%d vyeil '%s' lohu '%s.%d. %s' (%db/%db)", datum.den, datum.mesic, datum.rok, p_resitel->jmeno_resitele, p_reseni->p_uloha->p_archiv->jmeno_archivu, p_reseni->p_uloha->index_ulohy, p_reseni->p_uloha->jmeno_ulohy, p_reseni->body_reseni, p_reseni->p_uloha->body_ulohy);
    if (komentar != NULL) {
      printf("; koment '%s'\n", komentar);
    }
    else {
      printf("\n");
    }
  }
  // uvoln pam pro jmeno_archivu (u se nepouije)
  free(jmeno_archivu);

  // return the new structure
  return(p_reseni);
}

// Nate soubor se seznamem een a eitel a pid pslun odkazy do struktury loh i eitel
// Syntax souboru: {datum}{jmno eitele}{slo lohy}{body}
// Body mohou bt *
// Komente jsou uvozeny znaky '%', '#' nebo '/'
SEZNAM_RESENI * read_odpovedi(PARAMETERS * p_parameters, SEZNAM_ULOH * p_seznam_uloh, SEZNAM_RESITELU * p_seznam_resitelu) {
  SEZNAM_RESENI * seznam_reseni;               // seznam vech een
  FILE * soubor_resitele;                      // soubor 'resitele'
  RESENI * reseni;                             // naten een (odpov) ze souboru
  RESENI * predchozi_reseni;                   // pedchoz een (odpov) tho eitele
  char radek[MAX_RADEK];                       // pomocn string pro vstup dku
  char * sub_radek;                            // pozice ve stringu 'radek'
  CISLO_RADKU cislo_radku = 0;                 // poet dk v souboru vetn koment
  RESENA_ULOHA * p_resena_uloha;               // pointer na novou eenou lohu jednoho eitele
  RESENI_ULOHY * p_reseni_ulohy;               // pointer na nov een konkrtn lohy

  // Alokuj pam
  seznam_reseni = new_seznam_reseni();

  // Vypi zprvu co se dj
  if (p_parameters->verbosity_level > 2) {
    fprintf(stdout, "Otevrm soubor '%s' ...\n", p_parameters->in_file_resitele);
  }
  // otevi soubor odpovmi
  if ((soubor_resitele = fopen(p_parameters->in_file_resitele, "rt")) == NULL) {
    error_message(CANT_OPEN_FILE_RESITELE, p_parameters->in_file_resitele);
    //D    exit(CANT_OPEN_FILE_RESITELE);
  }

  // zpracuj soubor
  while (fgets(radek, MAX_RADEK, soubor_resitele) != NULL) {
    // nate jeden dek ze souboru, je-li pli dlouh (co nem bt) nape info
    if (radek[strlen(radek)-1] == '\n') {    // mme cel dal dek?
      cislo_radku++;
    }
    else {     // nebo byl pli dlouh?
      error_message(LINE_RESENI_TOO_LONG, radek);
      //D      exit(LINE_RESENI_TOO_LONG);
    }
    // pesko whitespaces
    sub_radek = radek;
    while (isspace(* sub_radek)) {
      sub_radek++;
    }
    // pesko skoro przdn dky (minimln "{}{}{}{}")
    if ((strlen(sub_radek)) <= VERY_SHORT_LINE_RESITELE) {
      continue;
    }
    // pesko dky s komentem (zanaj '%', '#' nebo '/')
    if ((* sub_radek == '%') || (* sub_radek == '#') || (* sub_radek == '/')) {
      continue;
    }
    // nati een ze zbytku dku
    reseni = nacti_reseni(p_parameters, p_seznam_uloh, p_seznam_resitelu, sub_radek);
    // dek soubory
    reseni->radek_souboru = cislo_radku;
    // nen nhodou znovu?
    if ((predchozi_reseni = find_predchozi_reseni(reseni->p_resitel, reseni->p_archiv->jmeno_archivu, reseni->p_uloha->index_ulohy)) != NULL) {
      fprintf(stdout, "'%s' vyeil lohu '%s.%d. %s' vcekrt!\n", reseni->p_resitel->jmeno_resitele, reseni->p_uloha->p_archiv->jmeno_archivu, reseni->p_uloha->index_ulohy, reseni->p_uloha->jmeno_ulohy);
      fprintf(stdout, "poprv: %d.%d.%d za %db s komentem '%s'\n", predchozi_reseni->datum.den, predchozi_reseni->datum.mesic, predchozi_reseni->datum.rok, predchozi_reseni->body_reseni, predchozi_reseni->komentar);
      fprintf(stdout, "podruh: %d.%d.%d za %db s komentem '%s'\n", reseni->datum.den, reseni->datum.mesic, reseni->datum.rok, reseni->body_reseni, reseni->komentar);
      error_message(VYRESENA_ULOHA_TWICE, reseni->p_uloha->jmeno_ulohy);
      //D      exit(VYRESENA_ULOHA_TWICE);
    }

    // zaa een do struktury seznamu een na zatek
    add_reseni(seznam_reseni, reseni);

    // zaa link na een do struktury eitele
    p_resena_uloha = new_resena_uloha(reseni);
    add_resena_uloha(reseni->p_resitel, p_resena_uloha);

    // zaa link na een do struktury archivu
    p_reseni_ulohy = new_reseni_ulohy(reseni);
    add_reseni_ulohy(reseni->p_uloha, p_reseni_ulohy);
  }

  // Bylo vbec njak een?
  if (seznam_reseni->n_reseni == 0) {
    error_message(NO_RESENI_IN_FILE_RESITELE, p_parameters->in_file_resitele);
  }
  // Zavi soubor s odpovmi
  if (fclose(soubor_resitele) != 0) {
    error_message(NOT_CLOSED_FILE_RESITELE, p_parameters->in_file_resitele);
    //D    exit(NOT_CLOSED_FILE_RESITELE);
  }
  if (p_parameters->verbosity_level > 2) {
    fprintf(stdout, "... zaveno. (nateno %ld een z %lu dk)\n", seznam_reseni->n_reseni, cislo_radku);
  }
  // done
  return seznam_reseni;
}

// eitel msce: k celkovho seznamu eitel vytvo zkrcen seznam eitel danho msce
SEZNAM_RESITELU * resitele_mesice(SEZNAM_RESITELU * p_seznam_resitelu, int mesic, int rok) {
  SEZNAM_RESITELU * p_seznam_resitelu_mesice;   // nov struktura SEZNAM_RESITELU
  RESITEL * p_resitel;                          // pointer pro prochzen seznamu eitel
  RESITEL * p_resitel_mesice;                   // nov struktura RESITEL
  RESENA_ULOHA * p_resena_uloha;                // pointer pro prochzen seznamu eench loh jednoho eitele
  RESENA_ULOHA * p_resena_uloha_mesice;         // nov struktura RESENA_ULOHA

  // alokuj pam
  p_seznam_resitelu_mesice = new_seznam_resitelu();

  // projdi vechny eitele
  p_resitel = p_seznam_resitelu->p_first_resitel;
  while (p_resitel != NULL) {
    // zatm nen eitelem msce
    p_resitel_mesice = NULL;
    // projdi vechny een danho eitele
    p_resena_uloha = p_resitel->p_first_resena_uloha;
    while (p_resena_uloha != NULL) {
      // je njak een z danho msce?
      if ((p_resena_uloha->p_reseni->datum.rok == rok) && (p_resena_uloha->p_reseni->datum.mesic == mesic)) {
        // ppadn zalo novho eitele msce
	if (p_resitel_mesice == NULL) {
	  p_resitel_mesice = new_resitel(p_resitel->jmeno_resitele);
	  // pidej ho do seznamu eitel msce
	  add_resitel(p_seznam_resitelu_mesice, p_resitel_mesice);
	}
        // zalo novou eenou lohu msce
	p_resena_uloha_mesice = new_resena_uloha(p_resena_uloha->p_reseni);
	// pidej ji do seznamu een danho eitele msce
	add_resena_uloha(p_resitel_mesice, p_resena_uloha_mesice);
      }
      // move on
      p_resena_uloha = p_resena_uloha->p_next_resena_uloha;
    }
    // move on
    p_resitel = p_resitel->p_next_resitel;
  }
  // pointer na novou strukturu
  return p_seznam_resitelu_mesice;
}

// Pikopruje soubor in_file (dan jmnem) do souboru out_file (dan file), chybov kdy pro error_open, error_close
STATUS include_file(FILE * out_file, char * jmeno_in_file, int error_code_open, int error_code_close) {
  FILE * in_file;                            // vstupn soubor
  char c;                                    // pro koprovn soubor

  // mme vbec out_file?
  if (out_file == NULL) {
    error_message(NO_OUTPUT_FILE, NULL);
    //D    exit(NO_OUTPUT_FILE);
  }
  // jak je input_file?
  if (jmeno_in_file == NULL) {
    error_message(NO_INPUT_FILE, NULL);
    //D    exit(NO_INPUT_FILE);
  }
  // oteve vstupn soubor
  if ((in_file = fopen(jmeno_in_file, "rt")) == NULL) {
    error_message(error_code_open, jmeno_in_file);
    //D    exit(error_code_open);
  }
  // kopiruje
  while ((c = fgetc(in_file)) != EOF) {
    putc(c, out_file);
  }
  // close hlavicka
  if (fclose(in_file) != 0) {
    error_message(error_code_close, jmeno_in_file);
    //D    exit(error_code_close);
  }
  // done
  return OK;
}


// Vytvo soubor archiv.php se seznamem loh rozdlench podle archiv
STATUS create_archive_file(PARAMETERS * parameters, SEZNAM_ULOH * p_seznam_uloh) {
  FILE * soubor_archiv;                      // vstupn soubor s archivem
  int n_archiv = 0;                          // poet archiv
  int n_uloh = 0;                            // poet loh
  ARCHIV * p_archiv;                         // jeden archiv v seznamu
  ULOHA * p_uloha;                           // pointer na lohu

  // Zprva co se dje
  if (parameters->verbosity_level > 2) {
    fprintf(stdout, "Generuji '%s' ...", parameters->out_file_archiv);
  }

  // Otevi soubor archiv.php
  if ((soubor_archiv = fopen(parameters->out_file_archiv, "wt")) == NULL) {
    error_message(CANT_OPEN_ARCHIV_FILE, parameters->out_file_archiv);
    //D    exit(CANT_OPEN_ARCHIV_FILE);
  }

  // hlavika
  include_file(soubor_archiv, parameters->archiv_head, CANT_OPEN_ARCHIV_HEADINGS, NOT_CLOSED_ARCHIV_HEADINGS);

  // TABLUKA LOH
  // hlavika tabulky
  fprintf(soubor_archiv, "<table border=\"1\" align=\"center\" class=\"tab\">\n");
  fprintf(soubor_archiv, "<tr class=\"tabhead\">\n");
  fprintf(soubor_archiv, " <th>Archiv</th>\n");
  fprintf(soubor_archiv, " <th>lohy</th>\n");
  fprintf(soubor_archiv, "</tr>\n");
  // zani od prvnho archivu
  p_archiv = p_seznam_uloh->p_first_archiv;
  // projdi cel seznam
  while (p_archiv != NULL) {
    if (p_archiv->typ_jmena_archivu == TYP_CISLA_STR) {
      // posu se na dal archiv
      p_archiv = p_archiv->p_next_archiv;
      continue;
    }
    // row heading
    n_archiv++;
    fprintf(soubor_archiv, "<tr class=\"tabrow%d\"><td class=\"archiv_number\"><a href=\"archiv%02d.php\">%d.&nbsp;archiv</a></td>\n", ((n_archiv-1) % 2), n_archiv, n_archiv);
    fprintf(soubor_archiv, "    <td class=\"archiv_ulohy\">");
    // zani od prvn lohy v archivu
    p_uloha = p_archiv->p_first_uloha;
    // po kadou lohu v archivu
    while (p_uloha != NULL) {
      // dal loha
      n_uloh += 1;
      if (parameters->info_body == 1) {  // vypi jmno a body
        fprintf(soubor_archiv, "%s", p_uloha->jmeno_ulohy);
      } else if (parameters->info_body == 2) {  // vypi jmno a body
        fprintf(soubor_archiv, "%s (%db)", p_uloha->jmeno_ulohy, p_uloha->body_ulohy);
      } else if (parameters->info_body == 3) {  // vypi jmno a body (i s hyperlinkem)
          if (p_uloha->p_archiv->typ_jmena_archivu == TYP_CISLA_STR) {
            fprintf(soubor_archiv, "<a href=\"ulohy.php#%s.%d.\">%s</a> (%db)", p_uloha->p_archiv->jmeno_archivu, p_uloha->index_ulohy, p_uloha->jmeno_ulohy, p_uloha->body_ulohy);
	  }
	  else {
            fprintf(soubor_archiv, "<a href=\"archiv%02d.php#%s.%d.\">%s</a> (%db)", p_uloha->p_archiv->cislo_archivu, p_uloha->p_archiv->jmeno_archivu, p_uloha->index_ulohy, p_uloha->jmeno_ulohy, p_uloha->body_ulohy);
	  }
      } else if (parameters->info_body == 4) {  // vypi slo lohy, jmno a body
        fprintf(soubor_archiv, "%s.%d. %s (%db)", p_uloha->p_archiv->jmeno_archivu, p_uloha->index_ulohy, p_uloha->jmeno_ulohy, p_uloha->body_ulohy);
      } else if (parameters->info_body >= 5) {  // vypi slo lohy, jmno a body a statistiku
        fprintf(soubor_archiv, "%s.%d. %s (%db) (%ld een, z toho %ld za pln poet)", p_uloha->p_archiv->jmeno_archivu, p_uloha->index_ulohy, p_uloha->jmeno_ulohy, p_uloha->body_ulohy, p_uloha->n_reseni, p_uloha->n_uplnych_reseni);
      }
      // oddl poloky rkou
      if (p_uloha->p_next_uloha != NULL) {
        fprintf(soubor_archiv, ", ");
      }
      // move on
      p_uloha = p_uloha->p_next_uloha;
    }
    // dek tabulky kon
    fprintf(soubor_archiv, "</td>\n</tr>\n");
    // pejdi na dal archiv
    p_archiv = p_archiv->p_next_archiv;
  }
  // konec tabulky
  fprintf(soubor_archiv, "<tr class=\"tabhead\"><td colspan=\"3\" align=\"center\">Tabulka obsahuje celkem %d loh v %d archivech.</td>\n</tr>\n", n_uloh, n_archiv);
  fprintf(soubor_archiv, "</table>\n");

  // patika
  include_file(soubor_archiv, parameters->archiv_tail, CANT_OPEN_ARCHIV_TAILS, NOT_CLOSED_ARCHIV_TAILS);

  // zavi soubor s archivem
  if (fclose(soubor_archiv) != 0) {
    error_message(NOT_CLOSED_ARCHIV_FILE, parameters->out_file_archiv);
    //D    exit(NOT_CLOSED_ARCHIV_FILE);
  }

  // Zprva o tom, co je udlno a kolik loh a archiv
  if (parameters->verbosity_level > 2) {
    fprintf(stdout, " dokoneno. (%d archiv obsahuje celkem %d loh)\n", n_archiv, n_uloh);
  }
  // done
  return OK;
}

// Vytvo tabulku eitel
POCET_RESENI create_tabulka_resitelu(PARAMETERS * parameters, SEZNAM_RESITELU * p_seznam_resitelu, FILE * soubor_resitele) {
  POCET_RESENI n_uloh_v_tabulce;             // poet loh v tabulce
  RESITEL * p_resitel;                       // pointer pro prochzen seznamu eitel
  RESENA_ULOHA * p_resena_uloha;             // pointer pro prochzen seznamu eench loh
  int row_count;                             // potadlo dk v tabulce

  // hlavika tabulky eitel
  fprintf(soubor_resitele, "<table border=\"1\" align=\"center\" class=\"tab\">\n");
  fprintf(soubor_resitele, "<tr class=\"tabhead\">\n");
  fprintf(soubor_resitele, " <th>Jmno</th>\n");
  fprintf(soubor_resitele, " <th>Poet bod</th>\n");
  fprintf(soubor_resitele, " <th>Vyeen lohy</th>\n");
  fprintf(soubor_resitele, "</tr>\n");
  // spotme poet loh v tabulce
  n_uloh_v_tabulce = 0;
  // Jestlie je seznam eitel przdn...
  if ((p_seznam_resitelu == NULL) || (p_seznam_resitelu->p_first_resitel == NULL) || (p_seznam_resitelu->n_resitelu == 0)) {
    fprintf(soubor_resitele, "<tr class=\"tabrow0\"><td colspan=\"3\" align=\"center\">Zatm nikdo ...</td>\n</tr>\n");
  }
  else {
    // zani prvnm eitelem v seznamu
    p_resitel = p_seznam_resitelu->p_first_resitel;
    // start row count with zero
    row_count = 0;
    // projdi seznam vech eitel
    while (p_resitel != NULL) {
      // zatek dku tabulky
      fprintf(soubor_resitele, "<tr class=\"tabrow%d\"><td class=\"resitel_name\">%s</td>\n", (row_count++ % 2), p_resitel->jmeno_resitele);
      fprintf(soubor_resitele, "    <td align=\"center\">%d</td>\n", p_resitel->suma_bodu);
      fprintf(soubor_resitele, "    <td class=\"resitel_ulohy\">");
      // projdi seznam loh, zani od prvn lohy
      p_resena_uloha = p_resitel->p_first_resena_uloha;
      // vypi kadou lohu tohoto eitele
      while (p_resena_uloha != NULL) {
	// dal loha v tabulce
	n_uloh_v_tabulce++;
	if (parameters->info_ulohy == 1) {  // vypi slo lohy, koment a body
	  fprintf(soubor_resitele, "%s.%d.%s (%db)", p_resena_uloha->p_reseni->p_uloha->p_archiv->jmeno_archivu, p_resena_uloha->p_reseni->p_uloha->index_ulohy, (p_resena_uloha->p_reseni->komentar == NULL) ? "" : p_resena_uloha->p_reseni->komentar, p_resena_uloha->p_reseni->body_reseni);
	}
	else if (parameters->info_ulohy == 2) {  // vypi jmno lohy, koment a body
	  fprintf(soubor_resitele, "%s%s (%db)", p_resena_uloha->p_reseni->p_uloha->jmeno_ulohy, (p_resena_uloha->p_reseni->komentar == NULL) ? "" : p_resena_uloha->p_reseni->komentar, p_resena_uloha->p_reseni->body_reseni);
	}
	else if (parameters->info_ulohy == 3) {  // vypi jmno lohy (i s odkazem!), koment a body
	  if (p_resena_uloha->p_reseni->p_uloha->p_archiv->typ_jmena_archivu == TYP_CISLA_STR) {
	    fprintf(soubor_resitele, "<a href=\"ulohy.php#%s.%d.\">%s</a>%s (%db)", p_resena_uloha->p_reseni->p_uloha->p_archiv->jmeno_archivu, p_resena_uloha->p_reseni->p_uloha->index_ulohy, p_resena_uloha->p_reseni->p_uloha->jmeno_ulohy, (p_resena_uloha->p_reseni->komentar == NULL) ? "" : p_resena_uloha->p_reseni->komentar, p_resena_uloha->p_reseni->body_reseni);
	  }
	  else {
	    fprintf(soubor_resitele, "<a href=\"archiv%02d.php#%s.%d.\">%s</a>%s (%db)", p_resena_uloha->p_reseni->p_uloha->p_archiv->cislo_archivu, p_resena_uloha->p_reseni->p_uloha->p_archiv->jmeno_archivu, p_resena_uloha->p_reseni->p_uloha->index_ulohy, p_resena_uloha->p_reseni->p_uloha->jmeno_ulohy, (p_resena_uloha->p_reseni->komentar == NULL) ? "" : p_resena_uloha->p_reseni->komentar, p_resena_uloha->p_reseni->body_reseni);
	  }
	}
	else if (parameters->info_ulohy == 4) {  // vypi slo i jmno lohy, koment a body
	  fprintf(soubor_resitele, "%s.%d. %s%s (%db)", p_resena_uloha->p_reseni->p_uloha->p_archiv->jmeno_archivu, p_resena_uloha->p_reseni->p_uloha->index_ulohy, p_resena_uloha->p_reseni->p_uloha->jmeno_ulohy, (p_resena_uloha->p_reseni->komentar == NULL) ? "" : p_resena_uloha->p_reseni->komentar, p_resena_uloha->p_reseni->body_reseni);
	}
	else if (parameters->info_ulohy == 5) {  // vypi slo i jmno lohy, koment a body een/body lohy
	  fprintf(soubor_resitele, "%s.%d. %s%s (%db/%db)", p_resena_uloha->p_reseni->p_uloha->p_archiv->jmeno_archivu, p_resena_uloha->p_reseni->p_uloha->index_ulohy, p_resena_uloha->p_reseni->p_uloha->jmeno_ulohy, (p_resena_uloha->p_reseni->komentar == NULL) ? "" : p_resena_uloha->p_reseni->komentar, p_resena_uloha->p_reseni->body_reseni, p_resena_uloha->p_reseni->p_uloha->body_ulohy);
	}
	// oddl jednotliv poloky rkou
	if (p_resena_uloha->p_next_resena_uloha != NULL) {
	  fprintf(soubor_resitele, ", ");
	}
	// pejdi na dal eenou lohu v seznamu
	p_resena_uloha = p_resena_uloha->p_next_resena_uloha;
      }
      // konec dku tabulky
      fprintf(soubor_resitele, "</td>\n</tr>\n");
      // dal eitel v seznamu
      p_resitel = p_resitel->p_next_resitel;
    }
    // patika tabulky
    fprintf(soubor_resitele, "<tr class=\"tabhead\"><td colspan=\"3\" align=\"center\">Tabulka obsahuje celkem %ld een od %d %s.</td>\n</tr>\n", n_uloh_v_tabulce, p_seznam_resitelu->n_resitelu, (p_seznam_resitelu->n_resitelu == 1) ? "eitele" : "eitel");
  }
  // konec tabulky
  fprintf(soubor_resitele, "</table>\n");
  //  return OK;
  return n_uloh_v_tabulce;
}

// Vytvo soubor resitele.php se seznamem eitel msce i celkovm poadm
STATUS create_resitele_file(PARAMETERS * parameters, SEZNAM_RESITELU * p_seznam_resitelu, SEZNAM_RESITELU * p_seznam_resitelu_mesice) {
  FILE * soubor_resitele;                    // vstupn soubor resitele.php
  POCET_RESENI n_uloh = 0;                   // poet eench loh celkem
  POCET_RESENI n_uloh_mesice = 0;            // poet eench loh tento msc

  // Zprva co se dje
  if (parameters->verbosity_level > 2) {
    fprintf(stdout, "Generuji '%s' ...", parameters->out_file_resitele);
  }

  // Otevi soubor resitele.php
  if ((soubor_resitele = fopen(parameters->out_file_resitele, "wt")) == NULL) {
    error_message(CANT_OPEN_RESITELE_FILE, parameters->out_file_resitele);
    //D    exit(CANT_OPEN_RESITELE_FILE);
  }

  // hlavika
  include_file(soubor_resitele, parameters->resitele_head, CANT_OPEN_RESITELE_HEADINGS, NOT_CLOSED_RESITELE_HEADINGS);

  // TABULKA MSCE
  fprintf(soubor_resitele, "<table width=\"98%%\">\n");
  fprintf(soubor_resitele, "<tr><td width=\"90%%\" align=\"center\"><h3 align=\"center\">Tabulka pnch eitel - %s %d</h3></td>\n", MESIC[parameters->datum.mesic], parameters->datum.rok);
  fprintf(soubor_resitele, "    <td align=\"right\"><img src=\"images/mince.gif\" width=\"48\" height=\"47\" alt=\"putovn cena aktulnho msce\" title=\"putovn cena aktulnho msce\" border=\"0\" /></td></tr>\n");
  fprintf(soubor_resitele, "</table>\n\n");
  //D  fprintf(soubor_resitele, "<h3 align=\"center\">Tabulka pnch eitel - %s %d</h3>\n\n", MESIC[parameters->datum.mesic], parameters->datum.rok);
  n_uloh_mesice = create_tabulka_resitelu(parameters, p_seznam_resitelu_mesice, soubor_resitele);

  // middle
  include_file(soubor_resitele, parameters->resitele_mids, CANT_OPEN_RESITELE_MIDS, NOT_CLOSED_RESITELE_MIDS);

  // TABULKA CELKOV
  n_uloh = create_tabulka_resitelu(parameters, p_seznam_resitelu, soubor_resitele);

  // patika 
  include_file(soubor_resitele, parameters->resitele_tail, CANT_OPEN_RESITELE_TAILS, NOT_CLOSED_RESITELE_TAILS);

  // zave soubor eitel
  if (fclose(soubor_resitele) != 0) {
    error_message(NOT_CLOSED_RESITELE_FILE, parameters->out_file_resitele);
    //D    exit(NOT_CLOSED_RESITELE_FILE);
  }
  // zprva o prbhu, kolik een
  if (parameters->verbosity_level > 2) {
    fprintf(stdout, " dokoneno. (%lu een celkem, %lu een tento msc)\n", n_uloh, n_uloh_mesice);
  }
  // done
  return OK;
}

// Vype info o aktulnch lohch
void aktualni_ulohy(PARAMETERS * parameters, SEZNAM_ULOH * p_seznam_uloh) {
  ARCHIV * p_archiv;                     // pointer na archiv v seznamu
  ULOHA * p_uloha;                       // pointer na lohu v seznamu

  // start with first 'loha' in first archive
  p_archiv = p_seznam_uloh->p_first_archiv;
  while (p_archiv != NULL) {
    if  ((p_archiv->typ_jmena_archivu == TYP_CISLA_STR) && (parameters->info_body >= 2)) {
      // info o aktulnch lohch
      fprintf(stdout, "Aktuln lohy:\n");
      // zani od prvn lohy
      p_uloha = p_archiv->p_first_uloha;
      while (p_uloha != NULL) {
        // info o jedn loze
        fprintf(stdout, "%s.%d. %s (%db) (%ld een", p_archiv->jmeno_archivu, p_uloha->index_ulohy, p_uloha->jmeno_ulohy, p_uloha->body_ulohy, p_uloha->n_uplnych_reseni);
	if (p_uloha->n_uplnych_reseni < p_uloha->n_reseni) {
	  fprintf(stdout, " + %ld dl", (p_uloha->n_reseni - p_uloha->n_uplnych_reseni));
	}
        fprintf(stdout, ")\n");
        // move on to the next 'loha'
        p_uloha = p_uloha->p_next_uloha;
      }
    }
    // move on
    p_archiv = p_archiv->p_next_archiv;
  }
  return;
}


/*
// sort 'eitel' by # of points
int sort_tabulka_resitelu_by_points(TABULKA_RESITELU * p_tabulka) {
  TABULKA_RESITELU usporadana_tabulka;      // sorted table
  RESITEL * p_sorted_resitel;               // 'eitel' in the sorted table
  RESITEL * p_resitel;                      // 'eitel' in the original table
  RESITEL * p_tmp_resitel;                  // for swapping purposes

  // first check if there is something to sort
  if ((p_tabulka == NULL) || (p_tabulka->p_first_resitel == NULL)) {
    // already sorted ;o)
    return OK;
  }
  // we start the same
  usporadana_tabulka.p_first_resitel = p_tabulka->p_first_resitel;
  p_resitel = p_tabulka->p_first_resitel->p_next_resitel;
  usporadana_tabulka.p_first_resitel->p_next_resitel = NULL;        // no more yet
  // go through the list one by one
  while (p_resitel != NULL) {
    p_sorted_resitel = usporadana_tabulka.p_first_resitel;
    // slip in at the beginning
    if (p_sorted_resitel->suma_bodu < p_resitel->suma_bodu) {
      p_tmp_resitel = p_resitel->p_next_resitel;
      p_resitel->p_next_resitel = p_sorted_resitel;
      usporadana_tabulka.p_first_resitel = p_resitel;
      p_resitel = p_tmp_resitel;
    }
    else {
      // find the correct place
      while ((p_sorted_resitel->p_next_resitel != NULL) && (p_sorted_resitel->p_next_resitel->suma_bodu >= p_resitel->suma_bodu)) {
	p_sorted_resitel = p_sorted_resitel->p_next_resitel;
      }
      // sort the 'eitel' in
      p_tmp_resitel = p_resitel->p_next_resitel;
      p_resitel->p_next_resitel = p_sorted_resitel->p_next_resitel;
      p_sorted_resitel->p_next_resitel = p_resitel;
      p_resitel = p_tmp_resitel;
    }
  }
  // we are done with it ;o)
  p_tabulka->p_first_resitel = usporadana_tabulka.p_first_resitel;
  return OK;
}

// sort 'eitel' by number of 'een loha'
int sort_tabulka_resitelu_by_ulohy(TABULKA_RESITELU * p_tabulka) {
  RESITEL * p_resitel;                      // eitel in the original table
  RESENA_ULOHA * p_resena_uloha;            // 'loha' in the original table
  RESENA_ULOHA * p_sorted_resena_uloha;     // 'loha' in the sorted table
  RESENA_ULOHA * p_tmp_resena_uloha;        // for swapping purposes

  // first check if there is something to sort
  if ((p_tabulka == NULL) || (p_tabulka->p_first_resitel == NULL)) {
    // already sorted ;o)
    return OK;
  }
  // sort every 'eitel'
  p_resitel = p_tabulka->p_first_resitel;
  while (p_resitel != NULL) {
    // sort the 'lohy'
    p_sorted_resena_uloha = p_resitel->p_first_resena_uloha;
    if (p_sorted_resena_uloha == NULL) {
      // this should never happen
      error_message(NO_RESENA_ULOHA_FOR_RESITEL, p_resitel->jmeno_resitele);
      exit(NO_RESENA_ULOHA_FOR_RESITEL);
    }
    // start with the first 'loha'
    p_resena_uloha = p_resitel->p_first_resena_uloha->p_next_resena_uloha;
    p_resitel->p_first_resena_uloha->p_next_resena_uloha = NULL;
    // insertsort every other
    while (p_resena_uloha != NULL) {
      p_sorted_resena_uloha = p_resitel->p_first_resena_uloha;
      // slip in at the beginning
      if (compare_ulohy_by_numbers(p_resena_uloha->p_uloha->cislo_ulohy, p_resitel->p_first_resena_uloha->p_uloha->cislo_ulohy) < 0) {
	p_tmp_resena_uloha = p_resena_uloha->p_next_resena_uloha;
	p_resena_uloha->p_next_resena_uloha = p_resitel->p_first_resena_uloha;
	p_resitel->p_first_resena_uloha = p_resena_uloha;
	p_resena_uloha = p_tmp_resena_uloha;
      }
      else {
        // find the correct place where to slip in
        while ((p_sorted_resena_uloha->p_next_resena_uloha != NULL) && (compare_ulohy_by_numbers(p_resena_uloha->p_uloha->cislo_ulohy, p_sorted_resena_uloha->p_next_resena_uloha->p_uloha->cislo_ulohy) > 0)) {
	  p_sorted_resena_uloha = p_sorted_resena_uloha->p_next_resena_uloha;
        }
	// sort the 'een loha' in
	p_tmp_resena_uloha = p_resena_uloha->p_next_resena_uloha;
	p_resena_uloha->p_next_resena_uloha = p_sorted_resena_uloha->p_next_resena_uloha;
	p_sorted_resena_uloha->p_next_resena_uloha  = p_resena_uloha;
	p_resena_uloha = p_tmp_resena_uloha;
      }
    }
    // move on to the next 'eitel'
    p_resitel = p_resitel->p_next_resitel;
  }
  return OK;
}

// sort 'eitel' by # of points
int counting_sort_tabulka_resitelu_by_points(TABULKA_RESITELU * p_tabulka) {
  RESITEL ** p_original_table;              // unsorted array
  RESITEL ** p_sorted_table;                // sorted array
  RESITEL * p_resitel;                      // 'eitel' in the original table
  int max_suma;                             // maximum of achieved points
  int * count;                              // array with point count
  int i, j;                                 // index variables

  //   // first check if there is something to sort
  if ((p_tabulka == NULL) || (p_tabulka->p_first_resitel == NULL) || (p_tabulka->p_first_resitel->p_next_resitel == NULL)) {
    // already sorted ;o)
    return OK;
  }

  // allocate memory for pointers as original array and sortet array
  p_original_table = (RESITEL **) malloc(p_tabulka->n_resitelu * sizeof(RESITEL *));
  p_sorted_table = (RESITEL **) malloc(p_tabulka->n_resitelu * sizeof(RESITEL *));

  // fill the original array and find maximum of points
  max_suma = 0;
  i = 0;
  p_resitel = p_tabulka->p_first_resitel;
  while (p_resitel != NULL) {
    // initialize original array
    p_original_table[i++] = p_resitel;
    // what is the maximum of points?
    if (p_resitel->suma_bodu > max_suma) {
      max_suma = p_resitel->suma_bodu;
    }
    // move on
    p_resitel = p_resitel->p_next_resitel;
  }
  // a quick check?
  if (i != p_tabulka->n_resitelu) {
    error_message(ERROR_IN_RESITEL_STRUCTURE, NULL);
    exit(ERROR_IN_RESITEL_STRUCTURE);
  }
  // allocate memory for array of point counts
  count = (int *) malloc((max_suma+1) * sizeof(int));
  // initialize the count array
  for (i=0; i <= max_suma; i++) {
    count[i] = 0;
  }
  // what are the counts?
  for (j=0; j < p_tabulka->n_resitelu; j++) {
    count[p_original_table[j]->suma_bodu]++;
  }
//D  // less or equal counts
//D  for (i=1; i <= max_suma; i++) {
//D    count[i] += count[i-1];
//D  }
//D  // set up the sorted table
//D  for (j=p_tabulka->n_resitelu-1; j >= 0; j--) { 
//D   p_sorted_table[(count[p_original_table[j]->suma_bodu]--)-1] = p_original_table[j];
//D  }
  // greater or equal counts
  for (i=max_suma-1; i >= 0; i--) {
    count[i] += count[i+1];
  }
  // set up the sorted table
  for (j=p_tabulka->n_resitelu-1; j >= 0; j--) {
    p_sorted_table[(count[p_original_table[j]->suma_bodu]--)-1] = p_original_table[j];
  }
//D  // set up the sorted table
//D  for (j=0; j < p_tabulka->n_resitelu; j++) {
//D    p_sorted_table[(count[p_original_table[j]->suma_bodu]--)-1] = p_original_table[j];
//D  }
  // make the sorted array ready
  if (p_tabulka->p_first_resitel != NULL) {
    p_tabulka->p_first_resitel = p_sorted_table[0];
    p_resitel = p_tabulka->p_first_resitel;
    for (j=1; j < p_tabulka->n_resitelu; j++) {
      p_resitel->p_next_resitel = p_sorted_table[j];
      // move on
      p_resitel = p_resitel->p_next_resitel;
    }
    // we are at the end
    p_resitel->p_next_resitel = NULL;
  }
  // free allocated memory
  free(p_original_table);
  free(p_sorted_table);
  free(count);
  return OK;
}

// sort 'eitel' by number of 'een loha'
int counting_sort_tabulka_resitelu_by_ulohy(TABULKA_RESITELU * p_tabulka) {
  RESITEL * p_resitel;                      // eitel in the original table
  RESENA_ULOHA * p_resena_uloha;            // 'loha' in the original table
  RESENA_ULOHA * p_sorted_resena_uloha;     // 'loha' in the sorted table
  RESENA_ULOHA * p_tmp_resena_uloha;        // for swapping purposes

  // first check if there is something to sort
  if (p_tabulka == NULL) {
    // already sorted ;o)
    return OK;
  }
  // sort 'lohy' for every 'eitel'
  p_resitel = p_tabulka->p_first_resitel;
  while (p_resitel != NULL) {
    // sort the 'lohy'
    p_sorted_resena_uloha = p_resitel->p_first_resena_uloha;
    if (p_sorted_resena_uloha == NULL) {
      // this should never happen
      error_message(NO_RESENA_ULOHA_FOR_RESITEL, p_resitel->jmeno_resitele);
      exit(NO_RESENA_ULOHA_FOR_RESITEL);
    }
    // nothing to sort
    if (p_resitel->pocet_uloh == 1) {
      continue;
    }
    // two can be sorted quickly
    else if (p_resitel->pocet_uloh == 2) {
      // sort two
      if (compare_ulohy_by_numbers(p_resitel->p_first_resena_uloha->p_next_resena_uloha->p_uloha->cislo_ulohy, p_resitel->p_first_resena_uloha->p_uloha->cislo_ulohy) < 0) {
	p_resitel->p_first_resena_uloha = p_resitel->p_first_resena_uloha->p_next_resena_uloha;
	p_resitel->p_first_resena_uloha->p_next_resena_uloha = p_sorted_resena_uloha;
	p_resitel->p_first_resena_uloha->p_next_resena_uloha->p_next_resena_uloha = NULL;
      }
      continue;
    }

    // ...

    // move on to the next 'eitel'
    p_resitel = p_resitel->p_next_resitel;
  }
  return OK;
}
*/

/*D
// nate jeden dek ze souboru, je-li pli dlouh (co nem bt) vrt LONG_LINE, jinak NORMAL_LINE
STATUS read_one_line(FILE * vstupni_soubor) { 
  //D PARAMETERS * p_parameters, 
  char radek[MAX_RADEK];           // pomocn string pro vstup dku
  char * sub_radek;                // pozice ve stringu 'radek'

  // nati dek
  if (fgets(radek, MAX_RADEK, vstupni_soubor) == NULL) {
    return NULL;
  } 
  // pesko whitespaces
  sub_radek = radek;
  while (isspace(* sub_radek)) {
    sub_radek++;
  }
  // almost empty line (at least "{}{}{}")
  if ((strlen(sub_radek)) <= 6) {
    continue;
  }
  // skip comment line (begins %, # or /)
  if ((* sub_radek == '%') || (* sub_radek == '#') || (* sub_radek == '/')) {
    continue;
  }
  return NORMAL_LINE;
}
*/

// ------------------------------------------------------------------------------------------------------------------------

// end of file
